
#include "cd3d.h"

cd3d::cd3d(HWND hWnd, UINT Width, UINT Height)
{
	g_pD3D=Direct3DCreate8(D3D_SDK_VERSION);
	g_pD3D->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &d3ddm);

	D3DPRESENT_PARAMETERS d3dpp; 
    ZeroMemory(&d3dpp, sizeof(d3dpp));

	d3dpp.AutoDepthStencilFormat = D3DFMT_D16;
	d3dpp.EnableAutoDepthStencil = TRUE; 

	d3dpp.Windowed = FALSE;//FALSE
    d3dpp.BackBufferCount = 1;
    d3dpp.BackBufferFormat = d3ddm.Format;
    d3dpp.BackBufferWidth = Width;
    d3dpp.BackBufferHeight = Height;
    d3dpp.hDeviceWindow = hWnd;
    d3dpp.SwapEffect = D3DSWAPEFFECT_COPY_VSYNC;
	d3dpp.FullScreen_RefreshRateInHz = D3DPRESENT_RATE_DEFAULT;//
    d3dpp.FullScreen_PresentationInterval = D3DPRESENT_INTERVAL_ONE;//

	g_pD3D->CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd, 
							D3DCREATE_SOFTWARE_VERTEXPROCESSING, &d3dpp, &g_pD3DDevice);

	g_pD3DDevice->SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR);
	g_pD3DDevice->SetTextureStageState(0, D3DTSS_MINFILTER, D3DTEXF_LINEAR);
	//g_pD3DDevice->SetRenderState(D3DRS_ZENABLE, D3DZB_TRUE); 
	g_pD3DDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
	g_pD3DDevice->SetRenderState(D3DRS_AMBIENT, D3DCOLOR_XRGB(180, 180, 180));
	g_pD3DDevice->SetRenderState(D3DRS_LIGHTING, TRUE);
	g_pD3DDevice->SetRenderState(D3DRS_NORMALIZENORMALS, TRUE);
	g_pD3DDevice->SetRenderState(D3DRS_FILLMODE, D3DFILL_SOLID);

	setMatrices();
}

void cd3d::RenderBegin()
{
    g_pD3DDevice->Clear(0, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER, D3DCOLOR_XRGB(0, 0, 0), 1.0f, 0);
    g_pD3DDevice->BeginScene();  
}

void cd3d::RenderEnd()
{
	g_pD3DDevice->EndScene();
    g_pD3DDevice->Present(NULL, NULL, NULL, NULL);
}

void cd3d::setMaterial(D3DCOLORVALUE rgbaDiffuse, D3DCOLORVALUE rgbaAmbient, D3DCOLORVALUE rgbaSpecular, D3DCOLORVALUE rgbaEmissive, float rPower)
{
	m_matMaterial.Diffuse = rgbaDiffuse; 
    m_matMaterial.Ambient = rgbaAmbient; 
    m_matMaterial.Specular = rgbaSpecular; 
    m_matMaterial.Power = rPower;
    m_matMaterial.Emissive = rgbaEmissive;

	g_pD3DDevice->SetMaterial(&m_matMaterial);
}

void cd3d::setLight()
{
	ZeroMemory(&d3dLight, sizeof(D3DLIGHT8));

	d3dLight.Type = D3DLIGHT_POINT;

	d3dLight.Diffuse.r = 1.0f;
	d3dLight.Diffuse.g = 1.0f;
	d3dLight.Diffuse.b = 1.0f;

	d3dLight.Ambient.r = 0.0f;
	d3dLight.Ambient.g = 0.0f;
	d3dLight.Ambient.b = 0.0f;

	d3dLight.Specular.r = 0.0f;
	d3dLight.Specular.g = 0.0f;
	d3dLight.Specular.b = 0.0f;

	d3dLight.Position.x = 18.0f;
	d3dLight.Position.y = 10.0f;
	d3dLight.Position.z = 18.0f;

	d3dLight.Attenuation0 = 1.0f;
	d3dLight.Attenuation1 = 0.0f;
	d3dLight.Attenuation2 = 0.0f;

	d3dLight.Range = 200.0f;

	g_pD3DDevice->SetLight(0, &d3dLight);
	g_pD3DDevice->LightEnable(0, TRUE);
}

void cd3d::setMatrices()
{
	D3DXMatrixLookAtLH( &matView, &D3DXVECTOR3( 0.0f, 120.0f, 150.0f ),
                                  &D3DXVECTOR3( 0.0f, 0.0f, 0.0f ),
                                  &D3DXVECTOR3( 0.0f, 1.0f, 0.0f ));

    g_pD3DDevice->SetTransform( D3DTS_VIEW, &matView );

    D3DXMatrixPerspectiveFovLH( &matProj, D3DX_PI/4, 1.0f, 1.0f, 500.0f );
    g_pD3DDevice->SetTransform( D3DTS_PROJECTION, &matProj );
}

void cd3d::CleanUp()
{
	if(g_pD3DDevice != NULL)
	{
        g_pD3DDevice->Release();
		g_pD3DDevice = NULL;
	}

    if(g_pD3D != NULL)
	{
        g_pD3D->Release();
		g_pD3D = NULL;
	}
}
