
#include <stdio.h>		//Hier haben wir ein paar Includes, welche wir fr
#include <d3dx8.h>		//den weitern Verlauf unseres Programmes gebrauchen knnen.
#include <string>		//z.B. zum ffnen von Dateien und Konvertieren von Variablen.
#include <fstream>			
#include <stdlib.h>
#include <math.h>

using namespace std;

#pragma comment(lib, "dxguid.lib")	//DirectX ist fr uns lebenswichtig.
#pragma comment(lib, "d3dx8.lib")

#define SafeRelease(pInterface) if(pInterface != NULL) {pInterface->Release(); pInterface=NULL;}
#define MDLVERTEX_FVF (D3DFVF_XYZ|D3DFVF_TEX1|D3DFVF_NORMAL)	//Unser Vertexformat.

typedef enum			//Die Animationstufen bekommen gewisse Integerwerte zugeordnet.
{						//Ist zwar nicht zwangsweise ntig, doch leichter zu verarbeiten und 
	BOTH_DEATH1 = 0,	//ausserdem bersichtlicher.
	BOTH_DEAD1,
	BOTH_DEATH2,
	BOTH_DEAD2,
	BOTH_DEATH3,
	BOTH_DEAD3,
	TORSO_GESTURE,
	TORSO_ATTACK,
	TORSO_ATTACK2,
	TORSO_DROP,	
	TORSO_RAISE,
	TORSO_STAND,
	TORSO_STAND2,
	LEGS_WALKCR,
	LEGS_WALK,
	LEGS_RUN,
	LEGS_BACK,
	LEGS_SWIM,
	LEGS_JUMP,
	LEGS_LAND,
	LEGS_JUMPB,
	LEGS_LANDB,
	LEGS_IDLE,
	LEGS_IDLECR,
	LEGS_TURN,
	MAX_ANIMATIONS
} eAnimations;

struct tAnimationInfo			//Unsere Animationen werden in dieser Struktur gespeichert.
{
	char strName[255];			//Dies speichert den Namen der Animation (z.B. "TORSO_STAND").
	int startFrame;				//Dies speichert die erste Framenummer der Animation.
	int endFrame;				//Dies speichert die letzte Framenummer der Animation.
	int loopingFrames;			//Dies speichert die looping Frames der ANimation (nicht benutzt)
	int framesPerSecond;		//Dies speichert die frames per second fr die Animation.
	int NextFrame;
	int Frame;

    double tLastFrame;			//Diese Werte sorgen dafr das alles immer gleich schnell luft,
    double tElapsed;			//egal auf welche PC.
    double tFactor;
};

// MD3Header speichert alle Informationen, welche am Anfang der MD3 stehen.
typedef struct
{
	char id[4];			//Die Datei ID - Muss "IDP3" sein.
	int  version;		//Die Dateiversion - Muss 15 sein.
	char filename[68];	//Name der Datei.
	int  numBoneFrames; //Zahl der Animationsframes.
	int  numTags;		//Taganzahl.
	int  numMeshes;		//Sub-Objekte in dem Mesh.
	int  numMaxSkins;	//Anzahl der Skins im Mesh.
	int  headerlength;	//Meshheader gre.
	int  tag_start;		//Der Einsatz in der Datei fr die Tags.
	int  tag_end;		//Der Endoffset in der Datei fr die Tags.
	int  filesize;		//Dateigre.
} MD3Header;

// Diese Struktur wird benutzt um die Meshinformationen aus den MD3's zu lesen.
typedef struct
{
	char id[4];			//Mesh ID.
	char name[68];		//Meshname.
	int  numMeshFrames;	//Meshanimation frame Anzahl.
	int  numSkins;		//Meshskin Anzahl.
	int  numVertices;	//Meshvertex Anzahl.
	int  numTriangles;	//Meshface count.
	int  tri_start;		//Start offset fr die triangles.
	int  headersize;	//Headergre fr das Mesh.
	int  texvec_start;	//Start offset fr die UV Koordinaten
	int  vertex_start;	//Start offset offset fr die Vertex Indices.
	int  meshsize;		//Meshgre.
} MeshHeader;

// Dies speichert den Skinnamen.
typedef struct
{
	char name[68];
} Skin;

// Triangle (face)
typedef struct 
{
	int vertex[3];
} Triangle;

// UV coordinates
typedef struct
{
	float texvec[2];
} TexCoord;

// Dies speichert die Normalen und Vertexindices.
typedef struct
{
	signed short  Vec[3];		//Der Vertex (Runterskaliert mit den Faktor 64.0f)
	unsigned char Normal[2];	//Die Noramlenwerte.
} Vertice;

//Dies speichert die Bone Informationen (Wird von uns nicht bentigt).
typedef struct
{
	float mins[3];			//Der Anfang der Bones (x, y, z).
	float maxs[3];			//Der Anfang der Bones (x, y, z).
	float position[3];		//Die Position des Bones.
	float scale;			//Die Skalierung des Bones.
	char  creator[16];		//Der Benutzte Modeler (z.B. "3DS MAX").
} BoneFrame;


//Dies ist unsere Struktur fr das MD3- Dateiformat. Dies wird benutzt um unsere 
//Models zu verbinden. Es Rotiert und Transliert sie.
typedef struct
{
	char name[64];			//Dies speichert den Namen des Tags (z.B. "tag_torso").
	float position[3];		//Dies speichert die Translation.
	float rotation[3][3];	//Dies speichert die 3x3 Rotationsmatrix fr den Frame.
} Tag;

typedef struct
{
	Vertice		*Vertices;
} Frame;


struct MD3Mesh
{
	MD3Header	MD3H;
	MeshHeader	MeshH;
	Skin		*Skins;
	Triangle	*Triangles;
	TexCoord	*TexCoords;
	Frame		*Frames;
	LPDIRECT3DTEXTURE8 texture;
	LPDIRECT3DVERTEXBUFFER8	*MD3VertexBuffer;
	unsigned char settexture; // 1 - On / 0 - Off
};

struct MDLVERTEX
{
    D3DVECTOR Position;
    D3DVECTOR Normal;
    float Texture[2];
};

struct MD3FrameVert
{
	MDLVERTEX				*MD3Verts;
};

struct MD3Vertice
{
	MD3FrameVert			*MD3FrameVerts;
};

struct MD3Model				//unser MD3Model enthlt die anschlissend wichtigen Infos.
{
	MD3Header				MD3H;
	MeshHeader				MeshH;

	BoneFrame				*BoneFrames;
	Tag						*Tags;

	MD3Vertice				*MD3Vertices;
	MD3Mesh					*MD3Meshes;
};

bool IsInString(string strString, string strSubString);	//Strings mssen verarbeitet werden.
double QPTimer();	//Wichtig fr die Framesynkronisation.
D3DXMATRIX Get4x4Matrix(MD3Model ModelPart, long TagID);	//Die 4x4 Matrix.

class cdMd3
{
	private:
		LPDIRECT3DDEVICE8		D3D_Device;			//Der Device unserer DX Anwendung.
		tAnimationInfo			animations[24];		//Die Animationen.
		tAnimationInfo			mUpper;				//Die Torsoanimation.
		tAnimationInfo			mLower;				//Die Beinanimatin.
		FILE					*fp;				//Dateivariable.
		D3DVECTOR				Normals[255][255];	//Die Noramlen fr den Character.
		int						TexNR;				//Benutzt um Skins zu laden.
		LPDIRECT3DTEXTURE8		TexInf[10];			//Texturen fr das Mesh.
		VOID					*pVertex;			//Fr den Vertexbuffer.

		D3DXMATRIX				matTorsoResult;		//Die Matrizen, welche zur Bearbeitung
		D3DXMATRIX				matHeadResult;		//fr das Mesh benutzt werden.
		D3DXMATRIX				matWeapon;
		D3DXMATRIX				matTemp;			//Sie gewinnen gerade beim rendern
		D3DXMATRIX				matWorld;			//an Bedeutung.
		D3DXMATRIX				ModelWorld;
		D3DXMATRIX				ModelWorld2;
		D3DXMATRIX				MatX;
		D3DXMATRIX				MatY;
		D3DXMATRIX				MatrixFrom;

		D3DVECTOR				PosFrom;

		MD3Model				MD3_Head;			//Unsere 3 Modelle. Ergeben sich aus 
		MD3Model				MD3_Upper;			//den drei Dateien.
		MD3Model				MD3_Lower;
		MD3Model				MD3_Weapon;			//Die Waffe.

	public:
		//Hier befinden sich smtliche, fr die Verarbeitung wichtigen, Methoden.
		//Wir werden sie in der folgenden cpp Datei genauer unter die Lupe nehmen.

		void Render(D3DXMATRIX matWorld);
		void ReadAnimationFile(LPSTR strConfigFile);
		void LoadMD3(LPSTR strModel);
		void ImportMD3(MD3Model *MD3, char *strFileName, char * strSkinName);
		void LoadSkin(MD3Model *MD3, char *filename);
		void DoMatrix(MD3Model *ModelPart, string Link, tAnimationInfo *MD3, D3DXMATRIX *ModelMatrix, bool IsHead);
		void SetLowerAni(long Ani);
		void SetUpperAni(long Ani);
		void UpdateFrames(tAnimationInfo *MD3);
		int SetMeshTexture(MD3Model *MD3, char *meshname, LPDIRECT3DTEXTURE8 tex);
		bool SetNormals();
		cdMd3(LPDIRECT3DDEVICE8 Device, LPSTR pFilename);
		~cdMd3();

		MD3Header				Header;
		BoneFrame				*BoneFrames;
		Tag						*Tags;
		MD3Mesh					*Meshes;
};
